@@ -61,23 +61,28 @@ class detrender:
         local_smoothing = median_filter(input=x, size=filter_size, mode=endrule_mode)
         return local_smoothing
 
+    # solution a bit janky but only median filter with changing window size?
+    def _detrend_global_runmed(self, x, filter_size):
+        x_series = pd.Series(x)
+        global_smoothing = x_series.rolling(filter_size, min_periods=1).median().to_numpy()
+        return global_smoothing
+
     def _detrend_lm(self, x, polynomial_degree):
-        x = np.linspace(1, x.size, x.size).astype(int).reshape((-1, 1))
+        y = np.linspace(1, x.size, x.size).astype(int).reshape((-1, 1))
         transformer = PolynomialFeatures(degree=polynomial_degree, include_bias=False)
-        data_ = transformer.fit_transform(x)
+        data_ = transformer.fit_transform(y)
         model = LinearRegression().fit(X=data_, y=x)
         predicted_value = model.predict(data_)
         return predicted_value
 
     def _run_detrend(self, x: np.ndarray) -> np.ndarray:
         if x.size:
-            local_smoothed = self._detrend_runnmed(x, self.smoothK, "constant")
+            local_smoothed = self._detrend_runnmed(x, self.smoothK, "nearest")
             if self.biasMet != "none":
                 if self.biasMet == "runmed":
-                    global_smoothed = self._detrend_runnmed(
+                    global_smoothed = self._detrend_global_runmed(
                         x=local_smoothed,
                         filter_size=self.biasK,
-                        endrule_mode="constant",
                     )
                 elif self.biasMet == "lm":
                     global_smoothed = self._detrend_lm(local_smoothed, self.polyDeg)
@@ -88,7 +93,7 @@ class detrender:
                     local_smoothed = np.divide(local_smoothed, local_smoothed.max())
                 local_smoothed = np.nan_to_num(local_smoothed)
         else:
-            local_smoothed = None
+            local_smoothed = np.array([])
 
         return local_smoothed
 
@@ -169,15 +174,24 @@ class binData(detrender):
         self.binThr = binThr
 
     def _rescale_data(self, x: np.ndarray, group_index: int, meas_index: int, feat_range: tuple = (0, 1)) -> np.ndarray:
-        grouped_array = np.split(x[meas_index, :], np.unique(x[group_index, :], axis=0, return_index=True)[1][1:])
-        out = [minmax_scale(i, feature_range=feat_range) for i in grouped_array]
+        meas_array = x[:, meas_index].astype('float64')
+        try:
+            group_array = x[:, group_index].astype('int64')
+        except ValueError:
+            try:
+                group_array = x[:, group_index].astype('float64')
+            except ValueError:
+                group_array = x[:, group_index].astype('U6')
+
+        grouped_array = np.split(meas_array, np.unique(group_array, axis=0, return_index=True)[1][1:])
+        out = [minmax_scale(i, feature_range=feat_range) if i.shape[0] > 0 else np.array([]) for i in grouped_array]
         rescaled = [item for sublist in out for item in sublist]
         # rescaled = minmax_scale(x[:,1], feature_range=feat_range)
         x[:, meas_index] = rescaled
         return x
 
     def _bin_data(self, x: np.ndarray) -> np.ndarray:
-        bin = (x > self.binThr).astype(np.int_)
+        bin = (x >= self.binThr).astype(np.int_)
         return bin
 
     def run(self, x: pd.DataFrame, colGroup: str, colMeas: str, colFrame: str) -> pd.DataFrame:
@@ -203,8 +217,13 @@ class binData(detrender):
         """
         col_resc = f"{colMeas}.resc"
         col_bin = f"{colMeas}.bin"
-        cols = [colGroup, colMeas]
+        col_fact = f'{colGroup}_factorized'
+        cols = [col_fact, colMeas]
         x.sort_values([colGroup, colFrame], inplace=True)
+        # factorize column in order to prevent numpy grouping error in detrending
+        value, label = x[colGroup].factorize()
+        x[col_fact] = value
+        # convert to numpy
         data_np = x[cols].to_numpy()
 
         if self.biasMet == "none":
@@ -215,13 +234,7 @@ class binData(detrender):
             detrended_data = self.detrend(data_np, group_index=0, meas_index=1)
             binarized_data = self._bin_data(detrended_data)
 
+        x = x.drop([col_fact], axis=1)
         x[col_resc] = detrended_data
         x[col_bin] = binarized_data
         return x
-
-
-if __name__ == '__main__':
-    df = pd.read_csv("/mnt/c/Users/benig/Downloads/objNuclei_1line_clean_tracks.csv")
-    df["col_meas"] = df["objCytoRing_Intensity_MeanIntensity_imKTR"] / df["objNuclei_Intensity_MeanIntensity_imKTR"]
-    ts = binData()
-    ts.run(df, 'track_id_uni', 'col_meas', 'Image_Metadata_T')
